package cz.vsb.krouzek;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

public class OknoSibenice extends JFrame {

	private static final long serialVersionUID = -5182233956382753964L;

	private static List<String> imageNames = Arrays.asList("sibenice-0.png", "sibenice-1.png", "sibenice-2.png",
			"sibenice-3.png", "sibenice-4.png", "sibenice-5.png", "sibenice-6.png", "sibenice-7.png", "sibenice-8.png");
	private JPanel letterPanel;
	private JPanel gamePanel;
	private JPanel midlePanel;
	private JButton startButton;
	private JButton endButton;
	private JLabel wordLabel;
	private JLabel infoLabel;
	private JLabel imageLabel;
	private List<JButton> charButtons = new ArrayList<JButton>();
	private int mistakeCount = 0;

	private Runnable startHry;
	private Runnable konecHry;
	private Consumer<List<Character>> vyberPismena;

	
	public OknoSibenice(Runnable startHry, Runnable konecHry, Consumer<List<Character>> vyberPismena){
		super("Hádání slov");
		this.startHry = startHry;
		this.konecHry = konecHry;
		this.vyberPismena = vyberPismena;
		initialize();
	}

	private void initialize() {
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		getContentPane().setLayout(new BorderLayout());

		// inicializace jednotlivých komponent přes getter metody
		getContentPane().add(getLetterPanel(), BorderLayout.SOUTH);
		getContentPane().add(getImageLabel(), BorderLayout.EAST);
		getContentPane().add(getGamePanel(), BorderLayout.NORTH);
		getContentPane().add(getMidlePanel(), BorderLayout.CENTER);

		pack();
		setVisible(true);
	}

	private JPanel getLetterPanel() {
		if (letterPanel == null) {
			letterPanel = new JPanel(new GridBagLayout());
			int counter = 1;
			for (Map.Entry<Character, List<Character>> entry : SibeniceZaklad.abeceda.entrySet()) {
				JButton button = new JButton(
						entry.getValue().stream().map(String::valueOf).collect(Collectors.joining(" ")));
				button.setName(String.valueOf(entry.getKey()));
				GridBagConstraints bagConstraints = new GridBagConstraints();
				bagConstraints.insets = new Insets(2, 2, 2, 2);
				if (counter % 9 == 0) {
					bagConstraints.gridwidth = GridBagConstraints.REMAINDER;
				}
				charButtons.add(button);
				button.addActionListener(e -> {
					String name = ((JButton)e.getSource()).getName();
					((JButton)e.getSource()).setEnabled(false);
					vyberPismena.accept(SibeniceZaklad.abeceda.get(name.charAt(0)));
				});
				letterPanel.add(button, bagConstraints);
				counter++;
			}
		}
		return letterPanel;
	}

	private JLabel getWordLabel() {
		if (wordLabel == null) {
			wordLabel = new JLabel("");
			wordLabel.setHorizontalAlignment(SwingConstants.CENTER);
			Font font =  new Font("Curier", Font.BOLD , 20);
			wordLabel.setFont(font);
			setWorldText("AB_H");
		}
		return wordLabel;
	}

	private JLabel getInfoLabel() {
		if (infoLabel == null) {
			infoLabel = new JLabel("");
			infoLabel.setHorizontalAlignment(SwingConstants.CENTER);
			Font font =  new Font("Curier", Font.BOLD , 20);
			infoLabel.setFont(font);
			infoLabel.setForeground(Color.RED);
			setWorldText("");
		}
		return infoLabel;
	}
	
	public void setInfoText(String text) {
		infoLabel.setText(text);
	}

	public void setWorldText(String text) {
		Pattern p = Pattern.compile("(.)");
		Matcher matcher = p.matcher(text);
		text = matcher.replaceAll("$1 ");
		getWordLabel().setText(text);
	}
	
	private JLabel getImageLabel() {
		if (imageLabel == null) {
			imageLabel = new JLabel("");
			updateImage();
		}
		return imageLabel;
	}

	public void setMistakeCount(int mistakeCount) {
		this.mistakeCount = mistakeCount;
		updateImage();
	}
	
	public int getMistakeCount() {
		return mistakeCount;
	}

	private void updateImage() {
		try {
			if(mistakeCount >= imageNames.size()) {
				mistakeCount = imageNames.size()-1;
			}
			BufferedImage image = ImageIO.read(this.getClass().getResourceAsStream("images/"+imageNames.get(mistakeCount)));
			imageLabel.setIcon(new ImageIcon(image));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private JButton getStartButton() {
		if (startButton == null) {
			startButton = new JButton("Start");
			startButton.addActionListener(e -> {
				charButtons.forEach(b -> b.setEnabled(true));
				startHry.run();
			});
		}
		return startButton;
	}

	private JButton getEndButton() {
		if (endButton == null) {
			endButton = new JButton("End");
			endButton.addActionListener(e -> {
				zakazTlacitkaPismenek(true);
				konecHry.run();
			});
		}
		return endButton;
	}

	public void zakazTlacitkaPismenek(boolean zakazat) {
		charButtons.forEach(b -> b.setEnabled(!zakazat));
	}

	private JPanel getMidlePanel() {
		if (midlePanel == null) {
			// Horní panel s tlačítky start a end
			midlePanel = new JPanel(new GridLayout(2,1));
			midlePanel.add(getWordLabel());
			midlePanel.add(getInfoLabel());
		}
		return midlePanel;
	}
	private JPanel getGamePanel() {
		if (gamePanel == null) {
			// Horní panel s tlačítky start a end
			gamePanel = new JPanel(new FlowLayout());
			gamePanel.add(getStartButton());
			gamePanel.add(getEndButton());
		}
		return gamePanel;
	}
}
